@@ -17,6 +17,11 @@
 #import "VLCDonationCreditCardViewController.h"
 #import "VLCStripeController.h"
 #import "VLCCurrency.h"
+#import "VLCPrice.h"
+#import "VLCSubscription.h"
+#import "VLCDonationInvoicesViewController.h"
+#import "VLCDonationSEPAViewController.h"
+#import "VLCSEPA.h"
 
 typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
 
@@ -25,11 +30,13 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     NSNumber *_selectedDonationAmount;
     NSArray *_availableCurrencies;
     VLCCurrency *_selectedCurrency;
+    VLCPrice *_selectedPrice;
+    VLCSubscription *_currentSubscription;
     BOOL _presentingCurrencySelector;
     VLCActionSheet *_actionSheet;
     PKPaymentButton *_applePayButton;
     UIImageView *_payPalImageView;
-    NSArray *_paymentProviders;
+    NSMutableArray *_paymentProviders;
     NSString *_selectedPaymentProvider;
     UIColor *_blueColor;
     UIColor *_lightBlueColor;
@@ -37,6 +44,10 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     NSString *_donationErrorMessage;
     VLCStripeController *_stripeController;
     CompletionHandler _successCompletionHandler;
+    BOOL _recurring;
+
+    NSArray <VLCPrice *> *_recurringPriceList;
+    NSArray *_monthlyButtonList;
 }
 @end
 
@@ -44,19 +55,21 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
 
 - (void)viewWillAppear:(BOOL)animated
 {
-    [self hidePurchaseInterface:NO];
     [super viewWillAppear:animated];
+    _stripeController.delegate = self;
+    [self hidePurchaseInterface:NO];
+    [_stripeController requestCurrentCustomerSubscription];
 }
 
 - (void)viewDidLoad {
     [super viewDidLoad];
 
-    _stripeController = [[VLCStripeController alloc] init];
+    _stripeController = [[VLCAppCoordinator sharedInstance] stripeController];
     _stripeController.delegate = self;
 
     /* use Euro as default currency and switch to a supported locale if available */
     NSLocale *locale = [NSLocale currentLocale];
-    NSString *currentLocaleCurrency = locale.currencyCode;
+    NSString *currentLocaleCurrency = [locale objectForKey:NSLocaleCurrencyCode];
     _selectedCurrency = [[VLCCurrency alloc] initEUR];
     _availableCurrencies = [VLCCurrency availableCurrencies];
     for (VLCCurrency *currency in _availableCurrencies) {
@@ -72,12 +85,19 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     }
     if (@available(iOS 14.0, *)) {
         self.continueButton.role = UIButtonRolePrimary;
+        self.monthlyUpdateButton.role = UIButtonRolePrimary;
+        self.monthlyCancelButton.role = UIButtonRoleCancel;
     }
 
     _titleLabel.text = NSLocalizedString(@"DONATION_TITLE", nil);
     _descriptionLabel.text = NSLocalizedString(@"DONATION_DESCRIPTION", nil);
     _customAmountField.placeholder = NSLocalizedString(@"DONATION_CUSTOM_AMOUNT", nil);
     [_continueButton setTitle:NSLocalizedString(@"DONATION_CONTINUE", nil) forState:UIControlStateNormal];
+    [_previousDonationsButton setTitle:NSLocalizedString(@"DONATION_INVOICES_RECEIPTS", nil) forState:UIControlStateNormal];
+    [_monthlyUpdateButton setTitle:NSLocalizedString(@"DONATION_CONTINUE", nil) forState:UIControlStateNormal];
+    [_monthlyCancelButton setTitle:NSLocalizedString(@"DONATION_CANCEL_MONTHLY", nil) forState:UIControlStateNormal];
+    [_intervalSelectorControl setTitle:NSLocalizedString(@"DONATION_ONCE", nil) forSegmentAtIndex:0];
+    [_intervalSelectorControl setTitle:NSLocalizedString(@"DONATION_MONTHLY", nil) forSegmentAtIndex:1];
 
     _selectedCurrencyButton.layer.cornerRadius = 5.;
     _continueButton.layer.cornerRadius = 5.;
@@ -87,6 +107,17 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     _thirtyButton.layer.cornerRadius = 5.;
     _fiftyButton.layer.cornerRadius = 5.;
     _hundredButton.layer.cornerRadius = 5.;
+    _monthlyFirstOptionButton.layer.cornerRadius = 5.;
+    _monthlySecondOptionButton.layer.cornerRadius = 5.;
+    _monthlyThirdOptionButton.layer.cornerRadius = 5.;
+    _monthlyUpdateButton.layer.cornerRadius = 5.;
+    _monthlyCancelButton.layer.cornerRadius = 5.;
+
+    _monthlyButtonList = @[_monthlyFirstOptionButton,
+                           _monthlySecondOptionButton,
+                           _monthlyThirdOptionButton];
+
+    self.monthlyPaymentView.hidden = YES;
 
     _actionSheet = [[VLCActionSheet alloc] init];
     _actionSheet.dataSource = self;
@@ -110,6 +141,9 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
                              object:nil];
 
     [self updateColors];
+
+    self.intervalSelectorControl.selectedSegmentIndex = 0;
+    [self segmentedControlAction:self];
 }
 
 - (void)updateColors
@@ -122,10 +156,11 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
 
     _selectedCurrencyButton.backgroundColor = colors.orangeUI;
     [_selectedCurrencyButton setTitleColor:whiteColor forState:UIControlStateNormal];
-    _continueButton.backgroundColor = colors.orangeUI;
+    _continueButton.backgroundColor = [UIColor grayColor];
     [_continueButton setTitleColor:whiteColor forState:UIControlStateNormal];
     _customAmountField.backgroundColor = colors.background;
     _customAmountField.layer.borderColor = colors.textfieldBorderColor.CGColor;
+    [_previousDonationsButton setTitleColor:colors.orangeDarkAccent forState:UIControlStateNormal];
 
     _fiveButton.backgroundColor = _lightBlueColor;
     [_fiveButton setTitleColor:whiteColor forState:UIControlStateNormal];
@@ -139,6 +174,13 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     [_fiftyButton setTitleColor:whiteColor forState:UIControlStateNormal];
     _hundredButton.backgroundColor = _lightBlueColor;
     [_hundredButton setTitleColor:whiteColor forState:UIControlStateNormal];
+
+    for (UIButton *button in _monthlyButtonList) {
+        button.backgroundColor = _lightBlueColor;
+        [button setTitleColor:whiteColor forState:UIControlStateNormal];
+    }
+    [_monthlyUpdateButton setTitleColor:whiteColor forState:UIControlStateNormal];
+    self.activityIndicatorView.color = colors.orangeUI;
 }
 
 - (void)adjustForKeyboard:(NSNotification *)aNotification
@@ -161,7 +203,7 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
         }
     }
 
-    [_contentScrollView scrollRectToVisible:_continueButton.frame animated:YES];
+    [_contentScrollView scrollRectToVisible:_previousDonationsButton.frame animated:YES];
 }
 
 - (NSString *)title
@@ -189,14 +231,20 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
 - (void)hidePurchaseInterface:(BOOL)bValue
 {
     self.selectedCurrencyButton.hidden = bValue;
-    self.fiveButton.hidden = bValue;
-    self.tenButton.hidden = bValue;
-    self.twentyButton.hidden = bValue;
-    self.thirtyButton.hidden = bValue;
-    self.fiftyButton.hidden = bValue;
-    self.hundredButton.hidden = bValue;
-    self.customAmountField.hidden = bValue;
-    self.continueButton.hidden = bValue;
+    self.intervalSelectorControl.hidden = bValue;
+    self.previousDonationsButton.hidden = bValue;
+    if (bValue) {
+        self.oneTimePaymentView.hidden = YES;
+        self.monthlyPaymentView.hidden = YES;
+    } else {
+        if (self.intervalSelectorControl.selectedSegmentIndex == 0) {
+            self.oneTimePaymentView.hidden = NO;
+            self.monthlyPaymentView.hidden = YES;
+        } else {
+            self.oneTimePaymentView.hidden = YES;
+            self.monthlyPaymentView.hidden = NO;
+        }
+    }
 }
 
 - (IBAction)switchCurrency:(id)sender
@@ -211,12 +259,15 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
 
 - (IBAction)numberButtonAction:(UIButton *)sender
 {
-    [self uncheckNumberButtons];
-    sender.selected = YES;
-    _selectedDonationAmount = [NSNumber numberWithInteger:sender.tag];
-    _continueButton.enabled = YES;
-    _applePayButton.enabled = YES;
-    sender.backgroundColor = _blueColor;
+    [UIView animateWithDuration:.25 animations:^{
+        [self uncheckNumberButtons];
+        sender.selected = YES;
+        self->_selectedDonationAmount = [NSNumber numberWithInteger:sender.tag];
+        self.continueButton.enabled = YES;
+        self.continueButton.backgroundColor = PresentationTheme.current.colors.orangeUI;
+        self->_applePayButton.enabled = YES;
+        sender.backgroundColor = self->_blueColor;
+    }];
 }
 
 - (IBAction)continueButtonAction:(id)sender
@@ -231,10 +282,184 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
 
 - (IBAction)customAmountFieldAction:(id)sender
 {
-    CGFloat floatValue = _customAmountField.text.floatValue;
-    _continueButton.enabled = floatValue > 0.;
-    _selectedDonationAmount = [NSNumber numberWithFloat:floatValue];
-    [self uncheckNumberButtons];
+    [UIView animateWithDuration:.25 animations:^{
+        CGFloat floatValue = self.customAmountField.text.floatValue;
+        self.continueButton.enabled = floatValue > 0.;
+        self.continueButton.backgroundColor = self.continueButton.enabled ? PresentationTheme.current.colors.orangeUI : [UIColor grayColor];
+        self->_selectedDonationAmount = [NSNumber numberWithFloat:floatValue];
+        [self uncheckNumberButtons];
+    }];
+}
+
+- (IBAction)showPreviousCharges:(id)sender
+{
+    VLCDonationInvoicesViewController *previousChargesVC = [[VLCDonationInvoicesViewController alloc] initWithNibName:nil bundle:nil];
+    [self.navigationController pushViewController:previousChargesVC animated:YES];
+}
+
+- (IBAction)segmentedControlAction:(id)sender
+{
+    _selectedDonationAmount = nil;
+    [UIView animateWithDuration:0.5 animations:^{
+        if (self.intervalSelectorControl.selectedSegmentIndex == 0) {
+            self.oneTimePaymentView.hidden = NO;
+            self.monthlyPaymentView.hidden = YES;
+            if (self->_selectedCurrency.supportsPayPal) {
+                if ([self->_paymentProviders indexOfObject:@"PayPal"] == NSNotFound) {
+                    [self->_paymentProviders addObject:@"PayPal"];
+                }
+            }
+            [self uncheckNumberButtons];
+            self->_continueButton.enabled = NO;
+            self->_continueButton.backgroundColor = [UIColor grayColor];
+        } else {
+            self.oneTimePaymentView.hidden = YES;
+            self.monthlyPaymentView.hidden = NO;
+            [self->_paymentProviders removeObject:@"PayPal"];
+            [self updateMonthlyButtons];
+        }
+    }];
+}
+
+#pragma mark - monthly donation actions
+
+- (void)setRecurringPriceList:(NSArray<VLCPrice *> *)priceList
+{
+    if (priceList.count != 3) {
+        APLog(@"Price list does not match expections");
+        [self.activityIndicatorView stopAnimating];
+        return;
+    }
+
+    _recurringPriceList = priceList;
+
+    NSNumberFormatter *formatter = [[NSNumberFormatter alloc] init];
+    formatter.numberStyle = NSNumberFormatterCurrencyStyle;
+    formatter.currencySymbol = _selectedCurrency.localCurrencySymbol;
+    formatter.maximumFractionDigits = 0;
+
+    [UIView animateWithDuration:0.5 animations: ^{
+        VLCPrice *price = priceList[0];
+        self.monthlyFirstOptionButton.hidden = NO;
+        [self.monthlyFirstOptionButton setTitle:[NSString stringWithFormat:NSLocalizedString(@"DONATION_MONTHLY_FORMAT", nil),
+                                             [formatter stringFromNumber:price.amount]]
+                                       forState:UIControlStateNormal];
+        [self.monthlyFirstOptionButton setTag:0];
+
+        price = priceList[1];
+        self.monthlySecondOptionButton.hidden = NO;
+        [self.monthlySecondOptionButton setTitle:[NSString stringWithFormat:NSLocalizedString(@"DONATION_MONTHLY_FORMAT", nil),
+                                              [formatter stringFromNumber:price.amount]]
+                                        forState:UIControlStateNormal];;
+        [self.monthlySecondOptionButton setTag:1];
+
+        price = priceList[2];
+        self.monthlyThirdOptionButton.hidden = NO;
+        [self.monthlyThirdOptionButton setTitle:[NSString stringWithFormat:NSLocalizedString(@"DONATION_MONTHLY_FORMAT", nil),
+                                             [formatter stringFromNumber:price.amount]]
+                                       forState:UIControlStateNormal];;
+        [self.monthlyThirdOptionButton setTag:2];
+
+        [self.activityIndicatorView stopAnimating];
+    }];
+    [self updateMonthlyButtons];
+}
+
+- (void)setCurrentSubscription:(VLCSubscription *)sub
+{
+    _currentSubscription = sub;
+    [self updateMonthlyButtons];
+}
+
+- (void)uncheckMonthlyButtons
+{
+    _monthlyFirstOptionButton.backgroundColor = _lightBlueColor;
+    _monthlySecondOptionButton.backgroundColor = _lightBlueColor;
+    _monthlyThirdOptionButton.backgroundColor = _lightBlueColor;
+}
+
+- (void)updateMonthlyButtons
+{
+    ColorPalette *colors = PresentationTheme.current.colors;
+    [self uncheckMonthlyButtons];
+
+    if (!_recurringPriceList)
+        return;
+
+    if (!_currentSubscription) {
+        self->_monthlyUpdateButton.enabled = NO;
+        self->_monthlyUpdateButton.backgroundColor = [UIColor grayColor];
+        [self->_monthlyUpdateButton setTitle:NSLocalizedString(@"DONATION_CONTINUE", nil) forState:UIControlStateNormal];
+        _monthlyCancelButton.backgroundColor = colors.background;
+        [_monthlyCancelButton setTitleColor:colors.lightTextColor forState:UIControlStateNormal];
+        return;
+    }
+
+    for (VLCPrice *price in _recurringPriceList) {
+        if ([price.id isEqualToString:_currentSubscription.priceid]) {
+            NSUInteger index = [_recurringPriceList indexOfObject:price];
+            UIButton *foundButton = _monthlyButtonList[index];
+            foundButton.backgroundColor = _blueColor;
+            [self setActiveSubscriptionState];
+            break;
+        }
+    }
+}
+
+- (void)setActiveSubscriptionState
+{
+    ColorPalette *colors = PresentationTheme.current.colors;
+    [UIView animateWithDuration:.25 animations:^{
+        self->_monthlyUpdateButton.enabled = YES;
+        self->_monthlyUpdateButton.backgroundColor = colors.orangeUI;
+        [self->_monthlyUpdateButton setTitle:NSLocalizedString(@"DONATION_UPDATE_MONTHLY", nil) forState:UIControlStateNormal];
+        self->_monthlyCancelButton.enabled = YES;
+        [self->_monthlyCancelButton setTitleColor:colors.orangeUI forState:UIControlStateNormal];
+    }];
+}
+
+- (IBAction)monthlyOptionAction:(UIButton *)sender
+{
+    [UIView animateWithDuration:.25 animations:^{
+        [self uncheckMonthlyButtons];
+        self->_selectedPrice = self->_recurringPriceList[sender.tag];
+        sender.backgroundColor = self->_blueColor;
+
+        self->_monthlyUpdateButton.enabled = YES;
+        self->_monthlyUpdateButton.backgroundColor = PresentationTheme.current.colors.orangeUI;
+    }];
+}
+
+- (IBAction)monthlyUpdateAction:(id)sender
+{
+    if (_currentSubscription == nil) {
+        [self continueButtonAction:sender];
+    } else {
+        [self.activityIndicatorView startAnimating];
+        [self hidePurchaseInterface:YES];
+        [_stripeController updateSubscription:_currentSubscription toPrice:_selectedPrice];
+    }
+}
+
+- (IBAction)monthlyCancelAction:(id)sender
+{
+    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"DONATION_VIDEOLAN",
+                                                                                                       comment: "")
+                                                                             message:NSLocalizedString(@"DONATION_CANCEL_LONG",
+                                                                                                       comment: "")
+                                                                      preferredStyle:UIAlertControllerStyleActionSheet];
+    [alertController addAction:[UIAlertAction actionWithTitle:NSLocalizedString(@"DONATION_CONTINUE", nil)
+                                                        style:UIAlertActionStyleDefault
+                                                      handler:^(UIAlertAction * _Nonnull action){
+    }]];
+    [alertController addAction:[UIAlertAction actionWithTitle:NSLocalizedString(@"BUTTON_CANCEL", nil)
+                                                        style:UIAlertActionStyleDestructive
+                                                      handler:^(UIAlertAction * _Nonnull action){
+        [self->_stripeController cancelSubscription:self->_currentSubscription];
+    }]];
+    alertController.popoverPresentationController.sourceView = self.monthlyCancelButton;
+
+    [self presentViewController:alertController animated:YES completion:nil];
 }
 
 #pragma mark - action sheet delegate
@@ -277,9 +502,16 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
 
 - (void)showSelectedCurrency
 {
+    [UIView animateWithDuration:0.5 animations:^{
+        [self.activityIndicatorView startAnimating];
+        self.monthlyFirstOptionButton.hidden = YES;
+        self.monthlySecondOptionButton.hidden = YES;
+        self.monthlyThirdOptionButton.hidden = YES;
+    }];
+    [_stripeController requestAvailablePricesInCurrency:_selectedCurrency];
     [self uncheckNumberButtons];
 
-    [_selectedCurrencyButton setTitle:_selectedCurrency.userReadableName forState:UIControlStateNormal];
+    [_selectedCurrencyButton setTitle:_selectedCurrency.localCurrencySymbol forState:UIControlStateNormal];
     NSArray *values = _selectedCurrency.values;
 
     NSNumberFormatter *formatter = [[NSNumberFormatter alloc] init];
@@ -300,27 +532,29 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     [_hundredButton setTitle:[formatter stringFromNumber:values[5]] forState:UIControlStateNormal];
     [_hundredButton setTag:[values[5] intValue]];
 
-    NSMutableArray *mutableProviders = [NSMutableArray array];
+    _paymentProviders = [NSMutableArray array];
     if (_selectedCurrency.supportsPayPal) {
-        [mutableProviders addObject:@"PayPal"];
+        [_paymentProviders addObject:@"PayPal"];
     }
-    // We depend on PKPaymentButtonTypeDonate introduced in iOS 10.2
-    if (@available(iOS 10.2, *)) {
-        // Check if Apple Pay is available
-        if ([PKPaymentAuthorizationViewController canMakePayments]) {
-            [mutableProviders addObject:@"Apple Pay"];
-        }
+    // Check if Apple Pay is available
+    if ([PKPaymentAuthorizationViewController canMakePayments]) {
+        [_paymentProviders addObject:@"Apple Pay"];
     }
     /* we need to support credit card authentication via 3D Secure for which we depend on
      * ASWebAuthenticationSession that was introduced in iOS 12 */
     if (@available(iOS 12.0, *)) {
-        [mutableProviders addObject:NSLocalizedString(@"DONATE_CC_DC", nil)];
+        [_paymentProviders addObject:NSLocalizedString(@"DONATE_CC_DC", nil)];
+    }
+    /* SEPA is available in EU, EFTA and 4 microstates for some currencies, if we have a valid translation for the legal contract
+     * As the UK is too complex after Brexit, it is ignored even SEPA is still supported with exceptions */
+    if ([VLCSEPA isAvailable] && _selectedCurrency.supportsSEPA) {
+        [_paymentProviders addObject:NSLocalizedString(@"DONATION_BANK_TRANSFER", nil)];
     }
-    _paymentProviders = [mutableProviders copy];
 }
 
 - (void)showSelectedPaymentProvider
 {
+    _recurring = self.intervalSelectorControl.selectedSegmentIndex != 0;
     if ([_selectedPaymentProvider isEqualToString:@"PayPal"]) {
         VLCDonationPayPalViewController *payPalVC = [[VLCDonationPayPalViewController alloc] initWithNibName:nil bundle:nil];
         [payPalVC setDonationAmount:_selectedDonationAmount.intValue];
@@ -330,8 +564,20 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
         [self initiateApplePayPayment];
     } else if ([_selectedPaymentProvider isEqualToString:NSLocalizedString(@"DONATE_CC_DC", nil)]) {
         VLCDonationCreditCardViewController *ccVC = [[VLCDonationCreditCardViewController alloc] initWithNibName:nil bundle:nil];
-        [ccVC setDonationAmount:_selectedDonationAmount withCurrency:_selectedCurrency];
+        if (_selectedPrice) {
+            [ccVC setPrice:_selectedPrice withCurrency:_selectedCurrency recurring:_recurring];
+        } else {
+            [ccVC setDonationAmount:_selectedDonationAmount withCurrency:_selectedCurrency];
+        }
         [self.navigationController pushViewController:ccVC animated:YES];
+    } else if ([_selectedPaymentProvider isEqualToString:NSLocalizedString(@"DONATION_BANK_TRANSFER", nil)]){
+        VLCDonationSEPAViewController *sepaVC = [[VLCDonationSEPAViewController alloc] initWithNibName:nil bundle:nil];
+        if (_selectedPrice) {
+            [sepaVC setPrice:_selectedPrice withCurrency:_selectedCurrency recurring:_recurring];
+        } else {
+            [sepaVC setDonationAmount:_selectedDonationAmount withCurrency:_selectedCurrency];
+        }
+        [self.navigationController pushViewController:sepaVC animated:YES];
     }
     _selectedPaymentProvider = nil;
 }
@@ -367,6 +613,11 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     VLCCurrency *currency = _availableCurrencies[indexPath.row];
     cell.name.text = currency.userReadableName;
     cell.name.textAlignment = NSTextAlignmentNatural;
+    if (currency == _selectedCurrency) {
+        cell.name.textColor = PresentationTheme.current.colors.orangeUI;
+    } else {
+        cell.name.textColor = PresentationTheme.current.colors.cellTextColor;
+    }
 }
 
 - (void)configureCellForPaymentProvider:(VLCActionSheetCell *)cell atIndexPath:(NSIndexPath *)indexPath
@@ -375,7 +626,11 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     cell.name.text = @"";
 
     if ([paymentProviderName isEqualToString:@"Apple Pay"]) {
-        _applePayButton = [PKPaymentButton buttonWithType:PKPaymentButtonTypeDonate style:PKPaymentButtonStyleBlack];
+        if (@available(iOS 10.2, *)) {
+            _applePayButton = [PKPaymentButton buttonWithType:PKPaymentButtonTypeDonate style:PKPaymentButtonStyleBlack];
+        } else {
+            _applePayButton = [PKPaymentButton buttonWithType:PKPaymentButtonTypePlain style:PKPaymentButtonStyleBlack];
+        }
         [cell addSubview:_applePayButton];
         _applePayButton.translatesAutoresizingMaskIntoConstraints = NO;
         /* This is a fake button, the action is handled by the containing collection view */
@@ -406,16 +661,61 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
     }
 }
 
+#pragma mark - stripe controller delegate
+
+- (void)customerSet
+{
+    [_stripeController requestCurrentCustomerSubscription];
+}
+
+- (void)stripeProcessingSucceeded
+{
+    [self.activityIndicatorView stopAnimating];
+    _donationSuccess = YES;
+    if (_successCompletionHandler) {
+        _successCompletionHandler(PKPaymentAuthorizationStatusSuccess);
+    } else {
+        [self donationReceived];
+    }
+}
+
+- (void)stripeProcessingFailedWithError:(NSString *)errorMessage
+{
+    _donationSuccess = NO;
+    _donationErrorMessage = errorMessage;
+    if (_successCompletionHandler) {
+        _successCompletionHandler(PKPaymentAuthorizationStatusFailure);
+    } else {
+        [self donationFailed];
+    }
+}
+
 #pragma mark - payment view controller delegate
 
 - (void)initiateApplePayPayment {
+    PKPaymentSummaryItem *summaryItem;
+    NSNumber *amount = _selectedPrice ? _selectedPrice.amount : _selectedDonationAmount;
+    if (_recurring) {
+        if (@available(iOS 15.0, *)) {
+            PKRecurringPaymentSummaryItem *summary = [PKRecurringPaymentSummaryItem summaryItemWithLabel:NSLocalizedString(@"DONATION_VIDEOLAN", "")
+                                                                                                  amount:[NSDecimalNumber decimalNumberWithDecimal:[amount decimalValue]]];
+            summary.intervalUnit = NSCalendarUnitMonth;
+            summary.intervalCount = 1;
+            summaryItem = summary;
+        } else {
+            summaryItem = [PKPaymentSummaryItem summaryItemWithLabel:NSLocalizedString(@"DONATION_MONTHLY_VIDEOLAN", "")
+                                                              amount:[NSDecimalNumber decimalNumberWithDecimal:[amount decimalValue]]];
+        }
+    } else {
+        summaryItem = [PKPaymentSummaryItem summaryItemWithLabel:NSLocalizedString(@"DONATION_VIDEOLAN", "")
+                                                          amount:[NSDecimalNumber decimalNumberWithDecimal:[amount decimalValue]]];
+    }
+
     PKPaymentRequest *paymentRequest = [[PKPaymentRequest alloc] init];
-    paymentRequest.countryCode = [[NSLocale currentLocale] countryCode];
+    paymentRequest.countryCode = [[NSLocale currentLocale] objectForKey:NSLocaleCountryCode];
     paymentRequest.merchantIdentifier = @"merchant.org.videolan.vlc";
     paymentRequest.merchantCapabilities = PKMerchantCapability3DS;
-    paymentRequest.paymentSummaryItems = @[
-        [PKPaymentSummaryItem summaryItemWithLabel:@"VideoLAN" amount:[NSDecimalNumber decimalNumberWithDecimal:[_selectedDonationAmount decimalValue]]]
-    ];
+    paymentRequest.paymentSummaryItems = @[summaryItem];
     paymentRequest.currencyCode = _selectedCurrency.isoCode;
     if (@available(iOS 12.0, *)) {
         paymentRequest.supportedNetworks = @[PKPaymentNetworkVisa, PKPaymentNetworkMasterCard, PKPaymentNetworkDiscover, PKPaymentNetworkAmex, PKPaymentNetworkMaestro];
@@ -434,6 +734,7 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
         [alertController addAction:[UIAlertAction actionWithTitle:NSLocalizedString(@"DONATION_CONTINUE", nil)
                                                             style:UIAlertActionStyleCancel
                                                           handler:nil]];
+        alertController.popoverPresentationController.sourceView = self.continueButton;
         [self presentViewController:alertController animated:YES completion:nil];
     }
 }
@@ -443,7 +744,11 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
                                 completion:(void (^)(PKPaymentAuthorizationStatus))completion
 {
     _successCompletionHandler = completion;
-    [_stripeController processPayment:payment forAmount:_selectedDonationAmount currency:_selectedCurrency];
+    [_stripeController processPayment:payment
+                            forAmount:_selectedDonationAmount
+                                price:_selectedPrice
+                             currency:_selectedCurrency
+                            recurring:_recurring];
 }
 
 - (void)paymentAuthorizationViewControllerDidFinish:(nonnull PKPaymentAuthorizationViewController *)controller {
@@ -451,36 +756,11 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
         if (self->_donationSuccess) {
             [self donationReceived];
         } else {
-            if (self->_donationErrorMessage != nil) {
-                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"PURCHASE_FAILED",
-                                                                                                                   comment: "")
-                                                                                         message:self->_donationErrorMessage
-                                                                                  preferredStyle:UIAlertControllerStyleActionSheet];
-                [alertController addAction:[UIAlertAction actionWithTitle:NSLocalizedString(@"BUTTON_OK", nil)
-                                                                    style:UIAlertActionStyleDefault
-                                                                  handler:^(UIAlertAction * _Nonnull action){
-                    [self dismissViewControllerAnimated:YES completion:nil];
-                }]];
-
-                [self presentViewController:alertController animated:YES completion:nil];
-            }
+            [self donationFailed];
         }
     }];
 }
 
-- (void)stripeProcessingSucceeded
-{
-    _donationSuccess = YES;
-    _successCompletionHandler(PKPaymentAuthorizationStatusSuccess);
-}
-
-- (void)stripeProcessingFailedWithError:(NSString *)errorMessage
-{
-    _donationSuccess = NO;
-    _donationErrorMessage = errorMessage;
-    _successCompletionHandler(PKPaymentAuthorizationStatusFailure);
-}
-
 - (void)donationReceived
 {
     [self hidePurchaseInterface:YES];
@@ -496,8 +776,27 @@ typedef void (^CompletionHandler)(PKPaymentAuthorizationStatus);
                                                       handler:^(UIAlertAction * _Nonnull action){
         [self dismissViewControllerAnimated:YES completion:nil];
     }]];
+    alertController.popoverPresentationController.sourceView = self.confettiView;
 
     [self presentViewController:alertController animated:YES completion:nil];
 }
 
+- (void)donationFailed
+{
+    if (self->_donationErrorMessage != nil) {
+        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"PURCHASE_FAILED",
+                                                                                                           comment: "")
+                                                                                 message:self->_donationErrorMessage
+                                                                          preferredStyle:UIAlertControllerStyleActionSheet];
+        [alertController addAction:[UIAlertAction actionWithTitle:NSLocalizedString(@"BUTTON_OK", nil)
+                                                            style:UIAlertActionStyleDefault
+                                                          handler:^(UIAlertAction * _Nonnull action){
+            [self dismissViewControllerAnimated:YES completion:nil];
+        }]];
+        alertController.popoverPresentationController.sourceView = self.confettiView;
+
+        [self presentViewController:alertController animated:YES completion:nil];
+    }
+}
+
 @end
